/***
 * OW2 FraSCAti Assembly Factory
 * Copyright (C) 2008-2009 INRIA, USTL
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: frascati@ow2.org 
 *
 * Author(s): Damien Fournier
 *            Christophe Demarey
 *            Philippe Merle
 * Contributor(s) : 
 */

package org.ow2.frascati.factory;

import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.LifeCycleController;
import org.objectweb.fractal.api.factory.Factory;
import org.objectweb.fractal.util.Fractal;
import org.ow2.frascati.factory.core.dispatcher.Manager;
import org.ow2.frascati.factory.core.instance.runtime.RuntimeFactory;
import org.ow2.frascati.tinfi.TinfiDomain;

/**
 * An abstract Factory defines the API for factories in charge of instantiating
 * SCA composite descriptions.
 */
public abstract class AbstractFactory {
	/**
	 * FRACTAL properties
	 */
	protected static final String FRACTAL_PROVIDER_PROPERTY = "fractal.provider";
	protected static final String FRACTAL_PROVIDERS_PROPERTY = "fractal.providers";
	protected static final String FRACTAL_PROVIDER_CHAINEDPROVIDER = "org.objectweb.fractal.util.ChainedProvider";
	protected static final String FRACTAL_PROVIDERS_TINFIJULIA = "org.objectweb.fractal.juliac.runtime.Juliac,org.objectweb.fractal.julia.Julia";

	/** Assembly factory instance */
	protected Component factory;

	/**
	 * Run the Factory with given arguments (composite path and the classpath).
	 * This method is simply called by the main method of the FactoryCommandLine
	 * tool.
	 * 
	 * @param factory
	 *            - The factory instance to use (AssemblyFactory,
	 *            RuntimeFactory, Generate Factory).
	 * @param compositeName
	 *            The name of the SCA composite to execute.
	 * @param serviceName
	 *            The name of the service to execute.
	 * @param methodName
	 *            The name of the method to execute.
	 * @param classpath
	 *            - List of URLs used to define the classpath.
	 */
	public static void run(AbstractFactory factory, String compositeName,
			String serviceName, String methodName, URL[] classpath) {
		// the SCA composite
		@SuppressWarnings("unused")
		Component scaDomain = null;

		// => Run the FraSCAti runtime
		// Set the RMI SecurityManager in order to allow dynamic class loading.
		// System.setSecurityManager( new java.rmi.RMISecurityManager() );
		ClassLoader cl = Thread.currentThread().getContextClassLoader();
		URL policyURL = cl.getResource("frascati.policy");
		System.setProperty("java.security.policy", policyURL.toString());

		// factory.launch(compositeName, "r", "run");
		if ((serviceName != null) && (methodName != null)) {
			scaDomain = factory.launch(compositeName, serviceName, methodName,
					classpath);
		} else {
			scaDomain = factory.getComposite(compositeName, classpath);
			System.gc();
			System.out.println("\nPress any key to exit");
			try {
				System.in.read();
			} catch (IOException e) {
				e.printStackTrace();
			}
		}

		System.out.println("\nExiting....");
		// TinfiDomain.close(scaDomain);

		System.exit(0);
	}

	/** Logger for this class */
	protected static Logger log;

	// Methods

	// Constructors
	public AbstractFactory() {
		log = Logger.getLogger(this.getClass().getCanonicalName());
		initializeFractal();
		factory = getFactory();
	}

	/**
	 * Close a SCA composite.
	 * 
	 * @param composite
	 *            The name of the SCA composite to close.
	 * @throws Exception
	 */
	public void close(Component composite) throws Exception {
		// Closing the SCA domain.
		try {
			log.info("Closing the SCA composite '" + composite + "'.");
			TinfiDomain.close(composite);
		} catch (Exception exc) {
			log.log(Level.SEVERE, "Impossible to stop the SCA composite '"
					+ composite + "'!", exc);
			return;
		}
	}

	/**
	 * Create an SCA assembly instance from composite file
	 * 
	 * @param composite
	 *            The composite file located into the current class path, on the
	 *            local file system, or at an URL
	 * 
	 * @return the SCA composite component or null if an error occurs.
	 */
	public Component getComposite(String composite) {
		return getComposite(composite, null);
	}

	/**
	 * Load jars and create an SCA assembly instance from composite file
	 * 
	 * @param composite
	 *            - The composite file located into the current class path, on
	 *            the local file system, or at an URL
	 * @param libs
	 *            - Libraries required to load this composite
	 * 
	 * @return the SCA composite component
	 */
	public Component getComposite(String composite, URL[] libs) {
		Manager factoryLoadService = null;
		// Component factory = getFactory();
		log.info("Starting the SCA composite : " + composite);

		try {
			factoryLoadService = ((Manager) factory.getFcInterface("load"));
		} catch (NoSuchInterfaceException e) {
			log
					.severe("Cannot retrieve the 'load' interface from the factory!");
			System.exit(-1);
		}

		if (libs != null)
			factoryLoadService.loadJars(libs);

		Component scaComposite = (Component) factoryLoadService
				.loadComposite(composite);

		start(scaComposite);

		return scaComposite;
	}

	/**
	 * Initialize the Assembly Factory component
	 * 
	 * @return The assembly factory component
	 */
	public Component getFactory() {

		Class<?> cl = null;
		Object o = null;
		Component c = null;

		if (factory != null) {
			return factory;
		} else {

			try {
				cl = Class.forName(getFactoryMainClass());
				o = cl.newInstance();
			} catch (Exception e) {
				log.severe("Cannot instantiate the factory :" + e.getMessage());
				System.exit(-1);
			}

			try {
				c = ((Factory) o).newFcInstance();
				Fractal.getLifeCycleController(c).startFc();
			} catch (Exception e) {
				log.severe("Cannot start the factory :" + e.getMessage());
				System.exit(-1);
			}
			o = null;
		}

		return c;
	}

	/**
	 * Get the factory main class.
	 */
	protected abstract String getFactoryMainClass();

	/**
	 * Initialize fractal properties.
	 */
	protected static void initializeFractal() {
		// Force fractal.provider and fractal.providers settings
		// TODO required by binding factory, remove if possible
		System.setProperty(FRACTAL_PROVIDER_PROPERTY,
				FRACTAL_PROVIDER_CHAINEDPROVIDER);
		System.setProperty(FRACTAL_PROVIDERS_PROPERTY,
				FRACTAL_PROVIDERS_TINFIJULIA);
	}

	/**
	 * Launch an SCA composite.
	 * 
	 * @param compositeName
	 *            The name of the SCA composite to execute.
	 * @param serviceName
	 *            The name of the service to execute.
	 * @param methodName
	 *            The name of the method to execute.
	 * @param jars
	 *            A list of URL for jar location
	 * 
	 * @return The composite component.
	 */
	public Component launch(final String compositeName,
			final String serviceName, final String methodName, URL[] jars) {

		// Loading the SCA composite.
		Component composite = getComposite(compositeName, jars);

		if (composite != null) {
			// Obtain the service.
			Object service = null;
			try {
				log.info("Obtaining the service '" + serviceName
						+ "' of the SCA composite '" + compositeName + "'...");
				service = TinfiDomain.getService(composite, Object.class,
						serviceName);
			} catch (NoSuchInterfaceException nsie) {
				log.log(Level.SEVERE, "The SCA composite '" + compositeName
						+ "' has no service '" + serviceName + "'!", nsie);
			}

			if (service != null) {
				try {
					log.info("Executing the method '" + methodName
							+ "' of the service '" + serviceName
							+ "' of the SCA composite '" + compositeName
							+ "'...");
					Method method = service.getClass().getMethod(methodName);
					method.invoke(service);
				} catch (NoSuchMethodException nsme) {
					log.log(Level.SEVERE, "The service '" + serviceName
							+ "' of the SCA composite '" + compositeName
							+ "' has no method '" + methodName + "'!", nsme);
				} catch (IllegalAccessException iae) {
					log.log(Level.SEVERE,
							"Impossible to execute the method '" + methodName
									+ "' of the service '" + serviceName
									+ "' of the SCA composite '"
									+ compositeName + "'!", iae);
				} catch (IllegalArgumentException iae) {
					log.log(Level.SEVERE,
							"Impossible to execute the method '" + methodName
									+ "' of the service '" + serviceName
									+ "' of the SCA composite '"
									+ compositeName + "'!", iae);
				} catch (InvocationTargetException ite) {
					log.log(Level.SEVERE,
							"Impossible to execute the method '" + methodName
									+ "' of the service '" + serviceName
									+ "' of the SCA composite '"
									+ compositeName + "'!", ite);
				}
			}
		}
		return composite;
	}

	/**
	 * Start an SCA composite by using its composite container
	 * 
	 * @param scaComposite
	 *            - Reference of the composite to start
	 * @return - composite status
	 */

	protected String start(Component scaComposite) {

		// WARNING : SCA composite must always has one and only one super
		// component
		// Which is the composite container.

		try {
			Component container[] = Fractal.getSuperController(scaComposite)
					.getFcSuperComponents();
			if (container.length != 1)
				throw new Error(
						"Composite container is not present or composite has more than one composite");

			LifeCycleController assemblyLcc = Fractal
					.getLifeCycleController(container[0]);
			assemblyLcc.startFc();

			log.info("SCA composite : " + assemblyLcc.getFcState() + "\n");

			return assemblyLcc.getFcState();

		} catch (Exception e) {
			throw new Error("Could not start this SCA composite", e);

		}
	}

	/**
	 * Set the URLClassLoader used by the Factory
	 * 
	 * @param classloader - URLClassLoader to be used by the Factory
	 */
	
	public void setClassLoader(URLClassLoader classloader) {
		try {
			RuntimeFactory runtime = (RuntimeFactory) getFactory()
					.getFcInterface("runtime");
			runtime.setClassLoader(classloader);

		} catch (NoSuchInterfaceException e) {
			e.printStackTrace();
		}
	}

	/**
	 * Get the ClassLoader used by the Factory
	 * 
	 * @return - ClassLoader currently used by the Factory
	 */
	
	public ClassLoader getClassLoader() {
		try {
			RuntimeFactory runtime = (RuntimeFactory) getFactory()
					.getFcInterface("runtime");

			return runtime.getClassLoader();

		} catch (NoSuchInterfaceException e) {
			e.printStackTrace();
		}
		return null;
	}

}
