/***
 * OW2 FraSCAti Assembly Factory
 * Copyright (C) 2008-2009 INRIA, USTL
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: frascati@ow2.org 
 *
 * Author: Damien Fournier
 */

package org.ow2.frascati.factory;

import java.io.File;
import java.net.URL;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.logging.Logger;

import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.IllegalContentException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.fractal.api.control.NameController;
import org.objectweb.fractal.api.factory.GenericFactory;
import org.objectweb.fractal.api.factory.InstantiationException;
import org.objectweb.fractal.api.type.ComponentType;
import org.objectweb.fractal.api.type.InterfaceType;
import org.objectweb.fractal.api.type.TypeFactory;
import org.ow2.frascati.factory.core.dispatcher.Manager;

/**
 * Domain Manager allows to load/unload severals SCA composite
 * 
 * @author <a href="mailto:damien.fournier@inria.fr">Damien Fournier</a>
 * @version $Revision$
 */
public class ScaDomainManager {
  /**
   * Logger for this class
   */
  Logger log = Logger.getLogger(ScaDomainManager.class.getCanonicalName());

  private AbstractFactory factory;

  @SuppressWarnings(value = "unused")
  private Component factoryComp;

  private Component domain;

  private HashMap<String, Component> componentRegistry = new HashMap<String, Component>();

  /**
   * Set factory to use by the SCA Domain
   * 
   * @param f
   *          - A FraSCAti factory instance
   */
  private void setFactory(AbstractFactory f) {
    factory = f;
    factoryComp = f.getFactory();
  }

  /**
   * Get Factory used for this domain
   * 
   * @return
   */
  public AbstractFactory getFactory() {
    return factory;
  }

  /**
   * Set the domain to manage represented as Fractal component
   * 
   * @param c
   *          - The Fractal component representing the domain
   * 
   */
  public void setDomain(Component c) {
    domain = c;
  }

  /**
   * SCA Domain constructor
   * 
   * @param f
   *          - The FraSCAti factory to associate with the domain
   */
  public ScaDomainManager(AbstractFactory f) {
    setFactory(f);
    try {
      Component bootstrap = org.objectweb.fractal.util.Fractal
          .getBootstrapComponent();
      GenericFactory gf = org.objectweb.fractal.util.Fractal
          .getGenericFactory(bootstrap);
      TypeFactory tf = org.objectweb.fractal.util.Fractal
          .getTypeFactory(bootstrap);
      ComponentType containerType = tf.createFcType(new InterfaceType[0]);
      Component d = gf.newFcInstance(containerType, "composite", null);
      NameController nc = org.objectweb.fractal.util.Fractal
          .getNameController(d);
      nc.setFcName("domain");
      setDomain(d);
    } catch (InstantiationException e) {
      e.printStackTrace();
    } catch (NoSuchInterfaceException e) {
      e.printStackTrace();
    }
  }

  /**
   * Load a new SCA composite
   * 
   * @param composite
   *          - Composite file name
   * @param jars
   *          - list of jars files to load
   * @throws NoSuchInterfaceException
   * @throws IllegalLifeCycleException
   * @throws IllegalContentException
   */
  public boolean load(String composite)
      throws NoSuchInterfaceException, IllegalContentException,
      IllegalLifeCycleException {
    Component c = factory.getComposite(composite);
    Component[] cList = org.objectweb.fractal.util.Fractal
        .getSuperController(c).getFcSuperComponents();
    for (Component component : cList) {
      org.objectweb.fractal.util.Fractal.getContentController(this.domain)
          .addFcSubComponent(component);
      this.componentRegistry.put(composite, component);
    }

    return true;

  }
  
  public boolean AddLibrary(URL lib) {
    Component factoryComponent = factory.getFactory();
    try {
      Manager factoryManagement = ((Manager) factoryComponent.getFcInterface("load"));
      factoryManagement.loadJars(new URL [] {lib});
    } catch (Exception e) {
      System.err.println("Could not load " + lib + " : FraSCAti Factory unaccessible");
    }
    return false;
  }
  

  /**
   * Unload an SCA composite
   * 
   * @param composite
   *          - composite name
   * @throws NoSuchInterfaceException
   * @throws IllegalLifeCycleException
   * @throws IllegalContentException
   */
  public void unload(String composite) throws NoSuchInterfaceException,
      IllegalContentException, IllegalLifeCycleException {
    Component component = this.componentRegistry.get(composite + "-container");
    try {
      org.objectweb.fractal.util.Fractal.getLifeCycleController(component)
          .stopFc();
      org.objectweb.fractal.util.Fractal.getContentController(domain)
          .removeFcSubComponent(component);
    } catch (NullPointerException e) {
      log.severe((("Composite " + composite) + " does not exist"));
    }
  }

  /**
   * Get list of SCA composites loaded in the Domain
   * 
   * @return
   */
  public List<java.lang.String> getLoadedComposites() {
    Component[] cList = null;
    List<java.lang.String> nList = new ArrayList<java.lang.String>();
    try {
      cList = org.objectweb.fractal.util.Fractal.getContentController(
          this.domain).getFcSubComponents();
      if (cList != null) {
        for (Component component : cList) {
          String name = org.objectweb.fractal.util.Fractal.getNameController(
              component).getFcName();
          System.out.println(name);
          nList.add(name.substring(0, ((name.length()) - 10)));
        }
      }
    } catch (NoSuchInterfaceException e) {
      e.printStackTrace();
    }
    return nList;
  }
  
  /**
   * Start composite of the Domain
   * 
   * @param composite
   *          - name of the composite
   * @return Status of the composite
   * @throws NoSuchInterfaceException
   * @throws IllegalLifeCycleException
   */
  public String startComposite(String composite)
      throws NoSuchInterfaceException, IllegalLifeCycleException {
    Component component = this.componentRegistry.get(composite + "-container");
    try {
      org.objectweb.fractal.util.Fractal.getLifeCycleController(component)
          .startFc();
      return org.objectweb.fractal.util.Fractal.getLifeCycleController(
          component).getFcState();
    } catch (NullPointerException e) {
      log.severe((("Composite " + composite) + " does not exist"));
    }
    return "Error";
  }

  /**
   * Stop composite of the Domain
   * 
   * @param composite
   *          - name of the composite
   * @return Status of the composite
   * @throws NoSuchInterfaceException
   * @throws IllegalLifeCycleException
   */
  public String stopComposite(String composite)
      throws NoSuchInterfaceException, IllegalLifeCycleException {
    Component component = this.componentRegistry.get(composite + "-container");
    try {
      org.objectweb.fractal.util.Fractal.getLifeCycleController(component)
          .stopFc();
      return org.objectweb.fractal.util.Fractal.getLifeCycleController(
          component).getFcState();
    } catch (NullPointerException e) {
      log.severe((("Composite " + composite) + " does not exist"));
    }
    return "Error";
  }
}
