/***
 * OW2 FraSCAti Assembly Factory
 * Copyright (C) 2008-2009 INRIA, USTL
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: frascati@ow2.org 
 *
 * Author: Damien Fournier
 */

package org.ow2.frascati.factory.core.instance.binding;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import org.eclipse.emf.common.util.EList;
import org.eclipse.stp.sca.Component;
import org.eclipse.stp.sca.ComponentReference;
import org.eclipse.stp.sca.ComponentService;
import org.eclipse.stp.sca.Composite;
import org.eclipse.stp.sca.Reference;
import org.eclipse.stp.sca.Service;
import org.eclipse.stp.sca.Wire;

import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.fractal.fraclet.annotations.Requires;
import org.objectweb.fractal.util.Fractal;

import org.ow2.frascati.factory.core.instance.NotSupportedException;

/**
 * @author <a href="mailto:damien.fournier@inria.fr">Damien Fournier</a>
 * @version $Revision$
 */

@org.objectweb.fractal.fraclet.annotations.Component
public class ScaBindingProcessImpl implements ScaBindingProcess {

  /**
   * Logger for this class
   */

  java.util.logging.Logger log = java.util.logging.Logger
      .getLogger(ScaBindingProcessImpl.class.getCanonicalName());

  @Requires(name = "binding-switch")
  ScaBinding sw;

  // indicates if binding switcher has been initialized
  // Needed until resolve compile problem when using Lyfecycle annotation
  boolean initBindingSwitch = false;

  /*
   * (non-Javadoc)
   * 
   * @see
   * org.ow2.frascati.factory.core.instance.binding.ScaBindingProcess#processBinding
   * (org.eclipse.stp.sca.Composite, org.objectweb.fractal.api.Component,
   * java.util.Map)
   */

  public void processBinding(Composite modelComposite,
      org.objectweb.fractal.api.Component fractalComposite,
      Map<String, org.objectweb.fractal.api.Component> map)
      throws NotSupportedException, BindingException {

    // Process component links defined as wires
    processWires(modelComposite, map);

    // Promote services of the composite
    exportComposite(modelComposite, fractalComposite, map);

    // Promote references of the composite
    importComposite(modelComposite, fractalComposite, map);

    // Process local/remote bindings of components
    bindComponents(modelComposite, map);

  }

  /**
   * Create bindings between components
   * 
   * @param modelComposite
   *          - The assembly according to the model definition
   * @param fractalComposite
   *          - The assembly according to fractal
   * @throws NotSupportedException
   * @throws BindingException
   */

  private void bindComponents(Composite modelComposite,
      Map<String, org.objectweb.fractal.api.Component> map)
      throws NotSupportedException, BindingException {

    // Cache for remote references
    HashMap<ComponentReference, org.objectweb.fractal.api.Component> bindMap = new HashMap<ComponentReference, org.objectweb.fractal.api.Component>();

    // list all components from the model
    EList<Component> componentList = modelComposite.getComponent();
    Iterator<Component> i = componentList.iterator();

    // for each component put reference and service data into maps
    while (i.hasNext()) {

      Component c = i.next();
      org.objectweb.fractal.api.Component fc = map.get(c.getName());

      EList<ComponentReference> refList = c.getReference();
      Iterator<ComponentReference> j = refList.iterator();

      // for each reference target
      while (j.hasNext()) {
        ComponentReference ref = j.next();
        // Put binding data into cache
        // Will create remote references after services
        bindMap.put(ref, fc);

        // if reference has target, must perform local binding for component
        // reference
        String target = ref.getTarget();

        if ((target != null) && (!target.equals(""))) {

          // The reference name
          String referenceName = ref.getName();
          // The targeted component name
          String tComponentName;
          // The targeted service name
          String tServiceName = null;

          String[] parameters = ref.getTarget().split("/");

          tComponentName = parameters[0];

          if (parameters.length > 1)
            tServiceName = parameters[1];
          else
            log.severe("Unknown targeted component for reference : "
                + referenceName);

          // retrieve target component
          org.objectweb.fractal.api.Component tComponent = map
              .get(tComponentName);

          localBinding(fc, referenceName, tComponent, tServiceName);

        }
      }

      EList<ComponentService> servList = c.getService();
      Iterator<ComponentService> k = servList.iterator();

      // for each service
      // process remote bindings defined in <binding.xxx> tag
      while (k.hasNext()) {
        sw.export(k.next(), fc);
      }

    }

    for (ComponentReference reference : bindMap.keySet()) {
      // process remote bindings defined in <binding.xxx> tag
      sw.bind(reference, bindMap.get(reference));
    }

  }

  /**
   * Create bindings between assembly references and components
   * 
   * @param modelComposite
   *          - The assembly according to the model definition
   * @param fractalComposite
   *          - The assembly according to fractal
   * @param map
   *          - Cache of composite's components referenced by their name
   * @throws NotSupportedException
   */

  private void importComposite(Composite modelComposite,
      org.objectweb.fractal.api.Component fractalComposite,
      Map<String, org.objectweb.fractal.api.Component> map)
      throws NotSupportedException {

    EList<Reference> references = modelComposite.getReference();

    for (Reference reference : references) {

      // The reference name
      String referenceName = reference.getName();
      // The promoted component name
      String tComponentName;
      // The promoted service name
      String tReferenceName;

      String promoted = reference.getPromote();
      String promotes[] = promoted.split("/");

      tComponentName = promotes[0];
      tReferenceName = promotes[1];

      // Get subcomponent using the component cache
      org.objectweb.fractal.api.Component subComponent = map
          .get(tComponentName);

      try {
        Fractal.getBindingController(subComponent).bindFc(
            tReferenceName,
            Fractal.getContentController(fractalComposite)
                .getFcInternalInterface(referenceName));
      } catch (NoSuchInterfaceException e) {
        e.printStackTrace();
      } catch (IllegalBindingException e) {
        e.printStackTrace();
      } catch (IllegalLifeCycleException e) {
        e.printStackTrace();
      }

      // process remote bindings
      sw.bind(reference, fractalComposite);

    }

  }

  /**
   * Create bindings between assembly services and components
   * 
   * @param modelComposite
   *          - The assembly according to the model definition
   * @param fractalComposite
   *          - The assembly according to fractal
   * @param map
   *          - Cache of composite's components referenced by their name
   * @throws NotSupportedException
   * @throws BindingException
   */

  private void exportComposite(Composite modelComposite,
      org.objectweb.fractal.api.Component fractalComposite,
      Map<String, org.objectweb.fractal.api.Component> map)
      throws NotSupportedException, BindingException {
    EList<Service> services = modelComposite.getService();

    for (Service service : services) {

      // The service name
      String serviceName = service.getName();
      // The promoted component name
      String tComponentName;
      // The promoted service name
      String tServiceName;

      String promoted = service.getPromote();
      String promotes[] = promoted.split("/");

      tComponentName = promotes[0];
      tServiceName = promotes[1];

      // create link implied by promote
      org.objectweb.fractal.api.Component subComponent = map
          .get(tComponentName);

      localBinding(fractalComposite, serviceName, subComponent, tServiceName);

      // process remote bindings
      sw.export(service, fractalComposite);

    }

  }

  /**
   * Perform local binding between two scaPrimitive components
   * 
   * @param source
   *          - Source component
   * @param sourceName
   *          - Source component interface name
   * @param target
   *          - Target component
   * @param targetName
   *          - Target component interface name
   * @throws BindingException
   */

  private void localBinding(org.objectweb.fractal.api.Component source,
      String sourceName, org.objectweb.fractal.api.Component target,
      String targetName) throws BindingException {

    try {
      Fractal.getBindingController(source).bindFc(sourceName,
          target.getFcInterface(targetName));
    } catch (Exception e) {
      try {
        String sc = Fractal.getNameController(source).getFcName();
        String tc = Fractal.getNameController(target).getFcName();
        throw new BindingException("Cannot bind  " + sc + "/" + sourceName
            + " with " + tc + "/" + targetName, e);
      } catch (Exception e2) {
        throw new BindingException(e);
      }
    }
  }

  /**
   * Process wires
   * 
   * @param modelComposite
   *          - The assembly according to the model definition
   * @throws BindingException
   */

  private void processWires(Composite modelComposite,
      Map<String, org.objectweb.fractal.api.Component> map)
      throws BindingException {

    // Get EMF List of Wires elements
    EList<Wire> wires = modelComposite.getWire();

    // Process each wires
    for (Wire wire : wires) {

      // Retrieve source component and its interface
      String source[] = wire.getSource().split("/");

      // Retrieve target component and its interface
      String target[] = wire.getTarget().split("/");

      // Test if source and target have been well defined
      if ((source.length == 2) && (target.length == 2)) {
        org.objectweb.fractal.api.Component targetComponent = map
            .get(target[0]);
        org.objectweb.fractal.api.Component sourceComponent = map
            .get(source[0]);

        // call for local binding between source and target components
        localBinding(sourceComponent, source[1], targetComponent, target[1]);

      } else
        throw new BindingException(
            "Wire error : source or target attribute must respect template : <ComponentName>/<ServiceName>");

    }

  }

}
