/***
 * OW2 FraSCAti Tinfi
 * Copyright (C) 2007-2009 INRIA, USTL
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: frascati@ow2.org
 *
 * Author: Lionel Seinturier
 */

package org.ow2.frascati.tinfi.juliac;

import org.objectweb.fractal.api.type.InterfaceType;
import org.objectweb.fractal.juliac.Juliac;
import org.objectweb.fractal.juliac.desc.MembraneDesc;
import org.objectweb.fractal.juliac.proxy.AbstractInterceptorSourceCodeGenerator;
import org.objectweb.fractal.juliac.ucf.UnifiedClass;
import org.objectweb.fractal.juliac.ucf.UnifiedMethod;
import org.objectweb.fractal.juliac.visit.MethodSourceCodeVisitor;
import org.osoa.sca.RequestContext;
import org.osoa.sca.annotations.EndsConversation;
import org.ow2.frascati.tinfi.TinfiComponentInterceptor;
import org.ow2.frascati.tinfi.osoa.RequestContextImpl;

/**
 * This class generates the source file of Tinfi interceptors which implement
 * the logic related to the management of component content instances. This
 * interceptor invokes the SCA content controller.
 *
 * @author Lionel Seinturier <Lionel.Seinturier@univ-lille1.fr>
 */
public class SCAContentInterceptorSourceCodeGenerator
extends AbstractInterceptorSourceCodeGenerator {
        
    public SCAContentInterceptorSourceCodeGenerator() {
        super();
    }

    public SCAContentInterceptorSourceCodeGenerator(
        InterfaceType it, MembraneDesc membraneDesc, boolean mergeable, Juliac jc ) {
        
        super(it,membraneDesc,mergeable,jc);
    }
    
    /**
     * Return the name of the suffix which is appended to generated class names.
     */
    public String getClassNameSuffix() {
        return "SCACC";
    }
        
    /**
     * Return true if some code must be generated by the current source file
     * generator.
     */
    public boolean match() {

        // Skip control interfaces
        String itname = it.getFcItfName();
        if( itname.equals("component") || itname.endsWith("-controller") ) {
            return false;
        }

        // Skip client interfaces
        if( it.isFcClientItf() ) {
            return false;
        }

        // Match all other interfaces (business server)
        return true;
    }

    @Override
    public void setMergeable( boolean mergeable ) {
        if(mergeable) {
            String msg = "This generator can not generate mergeable code";
            throw new IllegalArgumentException(msg);
        }
    }

    @Override
    public String getSuperClassName() {
        return TinfiComponentInterceptor.class.getName();
    }

    @Override
    public String[] getImplementedInterfaceNames() {
        return new String[]{it.getFcItfSignature()};
    }
    
    @Override
    public void generateProxyMethodBodyBeforeCode(
        MethodSourceCodeVisitor mv, UnifiedClass proxycl, UnifiedMethod proxym ) {
        
        // Create the request context
        mv.visit  ("    "+RequestContext.class.getName()+" rq = new ");
        mv.visit  (RequestContextImpl.class.getName());
        mv.visit  ("<"+it.getFcItfSignature()+">(\""+it.getFcItfName());
        mv.visitln("\","+it.getFcItfSignature()+".class,this);");
        
        // TODO the request context should be given a client proxy (not this which is a server one)
        
        /*
         * Push the request context on the thread local stack.
         */
        mv.visit  ("    java.util.Stack<"+RequestContext.class.getName());
        mv.visitln("> stack = getFcRequestContextStack();");
        mv.visitln("    stack.push(rq);");
        
        // Proceed with the call to the delegate
        mv.visitln("    "+it.getFcItfSignature()+" _impl = ("+it.getFcItfSignature()+") getFcContent();");
        
        // Begin of try/finally block
        mv.visitln("    try {");
    }
    
    @Override
    public void generateProxyMethodBodyFinallyCode(
        MethodSourceCodeVisitor mv, UnifiedClass proxycl, UnifiedMethod proxym ) {
        
        // End of try/finally block
        mv.visitln("    }");
        mv.visitln("    finally {");

        // Release the content instance
        mv.visit("      releaseFcContent(_impl,");
        if( proxym.getAnnotation(EndsConversation.class) == null ) {
            mv.visit("false");
        }
        else {
            mv.visit("true");
        }
        mv.visitln(");");
        
        // Pop the request context
        mv.visitln("      stack.pop();");        
        mv.visitln("    }");
    }
}
